﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading.Tasks;
using WatcherCommons.Interfaces;
using KinectProtocol.Properties;

namespace KinectProtocol
{
    public class KinectCamera : IWatcherCamera
    {
        private CameraConfiguration _dlgCfg = new CameraConfiguration();
        private Task _tCapture = null;

        private int _nResolution = 0;
        private bool _bCapturing = false;

        public KinectCamera()
        {
            _dlgCfg.Camera = this;
        }
        #region IWatcherCamera
        public event NewFrameEventHandler OnNewFrame = null;
        public Size FrameSize
        {
            get
            {
                return _nResolution == 0 ? Sensor.ColorFrameSize : Sensor.InfraredFrameSize;
            }
        }
        public string ConnectionString
        {
            get
            {
                return "type=" + (_nResolution == 0 ? "color" : "infrared");
            }
            set
            {
                _nResolution = 0;
                foreach (string s in value.Split(';'))
                {
                    string[] part = s.Split('=');
                    switch (part[0])
                    {
                        case "type":
                            _nResolution = part[1] == "color" ? 0 : 1;
                            break;
                    }
                }
            }
        }
        public string UserName { get; set; }
        public string Password { get; set; }
        public string Uri { get; set; }
        public int MaxFPS { get; set; }
        public bool Status
        {
            get
            {
                return _bCapturing;
            }
        }
        public ICameraSetupManager SetupManager
        {
            get
            {
                return _dlgCfg;
            }
        }
        public void Initialize()
        {
            Close();
            OnNewFrame = null;
        }
        public void ShowCameraConfiguration(Form parent)
        {
            _dlgCfg.Resolution = _nResolution;
            _dlgCfg.ReloadSettings();
            // Este cuadro de diálogo es no modal, ya que el usuario debe poder ver el resultado de los cambios
            // This dialog box is not modal, since the user should be able to see the result of changes
            _dlgCfg.Show();
            if (_nResolution != _dlgCfg.Resolution)
            {
                _nResolution = _dlgCfg.Resolution;
            }
        }
        public void Start()
        {
            // Ponemos en marcha el timer para capturar las imágenes
            // We start the timer to capture the images
            if (_tCapture == null)
            {
                _bCapturing = true;
                Sensor.OpenSensor(_nResolution == 0 ? SensorCaps.Color : SensorCaps.Infrared);
                _tCapture = Task.Run(()=> { CaptureThread(MaxFPS); });
            }
        }
        public void Close()
        {
            try
            {
                if (_tCapture != null)
                {                    
                    _bCapturing = false;
                    Task.WaitAll(new Task[] { _tCapture });
                    Sensor.CloseSensor();
                    _tCapture.Dispose();
                    _tCapture = null;
                }
            }
            catch
            {
            }
        }
        #endregion
        private void CaptureThread(object interval)
        {
            while (_bCapturing)
            {
                try
                {
                    DateTime tm = DateTime.Now;
                    Bitmap bmp = _nResolution == 0 ? Sensor.ColorFrame : Sensor.InfraredFrame;
                    if (bmp != null)
                    {
                        OnNewFrame?.Invoke(this, new FrameEventArgs(bmp));
                        TimeSpan ts;
                        do
                        {
                            ts = DateTime.Now.Subtract(tm);
                        } while (((ts.Seconds * 1000) + ts.Milliseconds) <= (int)interval);
                    }
                }
                catch
                {
                }
            }
        }
        public void SetResolution(int res)
        {
            _nResolution = res == 0 ? 0 : 1;
        }
        public override string ToString()
        {
            // Texto que se mostrará en la lista de cámaras
            // Text to be displayed in the camera list
            return Resources.ID_CAMPROTOCOL;
        }
    }
}
