﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using Microsoft.Kinect;

namespace KinectProtocol
{
    [Flags]
    public enum SensorCaps
    { 
        None = 0,
        Color = 1,
        Infrared = 2,
        Depth = 4,
        Body = 8
    }
    public static class Sensor
    {
        private static KinectSensor _sensor = null;
        private static MultiSourceFrameReader _reader = null;
        private static Bitmap _cFrame = null;
        private static Bitmap _iFrame = null;
        private static ushort[] _dFrame = null;
        private static Body [] _bodies = null;
        private static bool _bProcessing = false;
        private static int _instances = 0;
        private static Size _szColor = new Size(640, 480);
        private static Size _szInfrared = new Size(640, 480);
        private static string _sLock = "";

        public static SensorCaps Caps { get; set; }
        public static Size ColorFrameSize
        {
            get
            {
                return _szColor;
            }
            private set
            {
                _szColor = value;
            }
        }

        public static Size InfraredFrameSize
        {
            get
            {
                return _szInfrared;
            }
            private set
            {
                _szInfrared = value;
            }
        }

        public static bool BodyTracked
        {
            get
            {
                try
                {
                    if (_bodies != null)
                    {
                        lock (_sLock)
                        {
                            foreach (Body b in _bodies)
                            {
                                if (b.IsTracked)
                                {
                                    return true;
                                }
                            }
                        }
                    }
                    return false;
                }
                catch
                {
                    return false;
                }
            }
        }

        public static Bitmap ColorFrame
        {
            get
            {
                lock (_sLock)
                {
                    return _cFrame;
                }
            }
        }
        public static Bitmap InfraredFrame
        {
            get
            {
                lock (_sLock)
                {
                    return _iFrame;
                }
            }
        }
        public static ushort[] DepthFrame
        {
            get
            {
                lock (_sLock)
                {
                    return _dFrame;
                }
            }
        }
        public static void OpenSensor(SensorCaps caps)
        {
            Caps |= caps;
            if (_sensor == null)
            {
                _instances = 1;
                _sensor = KinectSensor.GetDefault();
                if (!_sensor.IsOpen)
                {
                    _sensor.Open();
                }
                Initialize();
                _reader = _sensor.OpenMultiSourceFrameReader(FrameSourceTypes.Color | FrameSourceTypes.Depth | FrameSourceTypes.Infrared | FrameSourceTypes.Body);
                _reader.MultiSourceFrameArrived += new EventHandler<MultiSourceFrameArrivedEventArgs>(OnNewFrame);
            }
            else
            {
                _instances++;
            }
        }
        public static void CloseSensor()
        {
            _instances--;
            if (_instances <= 0)
            {
                if (_sensor != null)
                {
                    if (_sensor.IsOpen)
                    {
                        _sensor.Close();
                        _reader.Dispose();
                        _reader = null;
                        _sensor = null;
                    }
                }
                _instances = 0;
                Caps = SensorCaps.None;
            }
        }
        private static void Initialize()
        {
            _bProcessing = false;
            _bodies = null;
            _cFrame = null;
            _iFrame = null;
            _dFrame = null;
            ColorFrameSize = new Size(_sensor.ColorFrameSource.FrameDescription.Width, _sensor.ColorFrameSource.FrameDescription.Height);
            InfraredFrameSize = new Size(_sensor.InfraredFrameSource.FrameDescription.Width, _sensor.InfraredFrameSource.FrameDescription.Height);
        }
        private static Bitmap CreateColorFrame(ColorFrame cframe)
        {
            Bitmap cbmp = null;
            BitmapData bd = null;
            try
            {
                if (cframe != null)
                {
                    try
                    {
                        cbmp = new Bitmap(cframe.FrameDescription.Width, cframe.FrameDescription.Height);
                        bd = cbmp.LockBits(new Rectangle(0, 0, cframe.FrameDescription.Width, cframe.FrameDescription.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
                        byte[] image = new byte[4 * cframe.FrameDescription.Width * cframe.FrameDescription.Height];
                        if (cframe.RawColorImageFormat == ColorImageFormat.Bgra)
                        {
                            cframe.CopyRawFrameDataToArray(image);
                        }
                        else
                        {
                            cframe.CopyConvertedFrameDataToArray(image, ColorImageFormat.Bgra);
                        }
                        Marshal.Copy(image, 0, bd.Scan0, image.Length);
                        cbmp.UnlockBits(bd);
                        bd = null;
                    }
                    catch
                    {
                    }
                    finally
                    {
                        if ((cbmp != null) && (bd != null))
                        {
                            cbmp.UnlockBits(bd);
                        }
                    }
                }
            }
            catch
            {
            }
            return cbmp;
        }
        private static ushort[] CreateDepthFrame(DepthFrame dframe)
        {
            ushort[] bw = null;
            try
            {
                if (dframe != null)
                {
                    try
                    {
                        bw = new ushort[dframe.FrameDescription.Width * dframe.FrameDescription.Height];
                        dframe.CopyFrameDataToArray(bw);
                    }
                    catch
                    {
                        bw = null;
                    }
                }
            }
            catch
            {
            }
            return bw;
        }
        private static Bitmap CreateInfraredFrame(InfraredFrame iframe)
        {
            Bitmap ibmp = null;
            BitmapData bd = null;
            try
            {
                if (iframe != null)
                {
                    try
                    {
                        ushort[] idata = new ushort[iframe.FrameDescription.Width * iframe.FrameDescription.Height];
                        iframe.CopyFrameDataToArray(idata);
                        ibmp = new Bitmap(iframe.FrameDescription.Width, iframe.FrameDescription.Height);
                        bd = ibmp.LockBits(new Rectangle(0, 0, iframe.FrameDescription.Width, iframe.FrameDescription.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
                        int[] ipixels = new int[iframe.FrameDescription.Width * iframe.FrameDescription.Height];
                        ushort imax = 0;
                        ushort imin = ushort.MaxValue;
                        for (int ix = 0; ix < idata.Length; ix++)
                        {
                            imax = Math.Max(imax, idata[ix]);
                            imin = Math.Min(imin, idata[ix]);
                        }
                        int rec = imax - imin;
                        for (int ix = 0; ix < idata.Length; ix++)
                        {
                            int pxint = (255 * (idata[ix] - imin)) / rec;
                            ipixels[ix] = Color.FromArgb(pxint, pxint, pxint).ToArgb();
                        }
                        Marshal.Copy(ipixels, 0, bd.Scan0, ipixels.Length);
                        ibmp.UnlockBits(bd);
                        bd = null;
                    }
                    catch
                    {
                    }
                    finally
                    {
                        if ((ibmp != null) && (bd != null))
                        {
                            ibmp.UnlockBits(bd);
                        }
                    }
                }
            }
            catch
            {
            }
            return ibmp;
        }
        private static void OnNewFrame(object sender, MultiSourceFrameArrivedEventArgs e)
        {
            if (!_bProcessing)
            {
                try
                {
                    _bProcessing = true;
                    MultiSourceFrame frm = e.FrameReference.AcquireFrame();
                    if (frm != null)
                    {
                        lock (_sLock)
                        {
                            if (Caps.HasFlag(SensorCaps.Body))
                            {
                                using (BodyFrame bframe = frm.BodyFrameReference.AcquireFrame())
                                {
                                    if (bframe != null)
                                    {
                                        _bodies = new Body[bframe.BodyCount];
                                        bframe.GetAndRefreshBodyData(_bodies);
                                    }
                                }
                            }
                            if (Caps.HasFlag(SensorCaps.Color))
                            {
                                using (ColorFrame cframe = frm.ColorFrameReference.AcquireFrame())
                                {
                                    _cFrame = CreateColorFrame(cframe);
                                }
                            }
                            if (Caps.HasFlag(SensorCaps.Depth))
                            {
                                using (DepthFrame dframe = frm.DepthFrameReference.AcquireFrame())
                                {
                                    _dFrame = CreateDepthFrame(dframe);
                                }
                            }
                            if (Caps.HasFlag(SensorCaps.Infrared))
                            {
                                using (InfraredFrame iframe = frm.InfraredFrameReference.AcquireFrame())
                                {
                                    _iFrame = CreateInfraredFrame(iframe);
                                }
                            }
                        }
                    }
                }
                catch
                {
                }
                finally
                {
                    _bProcessing = false;
                }
            }
        }
    }
}

